MGT  = {}
MGT.Version = '2.0.7'

--[[



]]--



--###################################################################################################################################################
-- #############	DCS - Mission Generator  Tool																						################# 
-- #############	by Draken35																										################# 
--###################################################################################################################################################
--[[ Requirements:	
		>>>	 MIST 4.5.107 or above <https://github.com/mrSkortch/MissionScriptingTools/tree/master>
	
--]]
-------

--###################################################################################################################################################
-- #############   Configuration section   																							################# 
--###################################################################################################################################################


MGT.DistanceUnitsDefault		= 1    	-- 1 = Nautical Miles, 2 = Miles, 3 = Kilometers
MGT.AltitudeUnitsDefault		= 1    	-- 1 = feet, 3 = meters ( 2 is not used)
MGT.CoordinateFormatDefault		= 2	   	-- 1 = MGRS, 2 = DMS, 3 = DM.mm
MGT.Era							= 2 	-- 2 = Modern , 1 = Cold war


MGT.messageBeep = '204521__redoper__roger-beep.ogg' -- Message alert sound. leave empty for no sound


--#################################################################################################################################################
-- #############   Initialization of globals (infrastructure)																		############### 
--#################################################################################################################################################

MGT.populatedUnitPositions = {}
MGT.clientConfig = {}
MGT.globalConfig = {
					BlueRootPath = nil
				,	RedRootPath = nil	
				,	nextID = 0
			}

MGT.templateList = {}
MGT.CardinalDirections = {
							{deg =    0, name = 'North'},
							{deg =   45, name = 'North-East'},
							{deg =   90, name = 'East'},
							{deg =  135, name = 'South-East'},
							{deg =  180, name = 'South'},
							{deg =  225, name = 'South-West'},
							{deg =  270, name = 'West'},
							{deg =  315, name = 'North-West'}
						}

-----------------------------------------------------------------------------------------------------------------------------------------------------

--#################################################################################################################################################
-- #############   code	lib													 	  													############### 
--#################################################################################################################################################
function MGT.Settings(_args)
	local unitName = _args[1]
	local setting = _args[2]
	local cfg = MGT.clientConfig[unitName]

	if setting ==  0 then -- Display settings
		local _msg  = ''
		
		
		_msg = string.format ('Flying: %s',cfg.unitType)
		-- 1 = Nautical Miles, 2 = Miles, 3 = Kilometers
		if MGT.Era == 1 then -- Modern
			_msg = string.format ('%s\nModern Era',_msg)
		elseif MGT.Era == 2 then --CW
			_msg = string.format ('%s\nCold War Era',_msg)
		end 
		if MGT.MarkAllSams	then
			_msg = string.format ('%s\nMark all SAMs: ON',_msg)
		else
			_msg = string.format ('%s\nMark all SAMs: OFF',_msg)
		end
		
		local cat = ''
		local o = MGT.MissionCategory
		if o == 0 then 
			cat ="Any"		
		elseif o == 1 then 
			cat ="Interception (A2A)"
		elseif o == 2 then 			
			cat ="Strike (A2G static structures)"	
		elseif o == 3 then 			
			cat ="Close Air Support(JTAC)"		
		elseif o == 4 then 			
			cat ="Interdiction (A2G units)"		
		elseif o == 5 then 			
			cat ="Anti-Ship"					
		elseif o == 6 then 			
			cat ="SEAD"						
		elseif o == 7 then 			
			cat ="CAP"							
		end
		_msg = string.format ('%s\nCategory: %s',_msg, cat)
	
		MGT.messageToUnit(unitName,_msg,{},10)
	

	-- Distance
	-- 1 = Nautical Miles, 2 = Miles, 3 = Kilometers
	elseif setting == 1 then -- Distance km
		MGT.Era = 1
		MGT.messageToUnit(unitName,'Modern Era')
	elseif setting == 2 then --Distance  nm
		MGT.Era = 2
		MGT.messageToUnit(unitName,'Cold War Era')
	elseif setting == 3 then
		MGT.Density = 0.5
		MGT.messageToUnit(unitName,'Decreased AD density')
	elseif setting == 4 then
		MGT.Density = 1.0
		MGT.messageToUnit(unitName,'Normal AD density')
	elseif setting == 5 then
		MGT.Density = 1.5
		MGT.messageToUnit(unitName,'Increased AD density')	
	elseif setting == 6 then
		MGT.MarkAllSams	= true	
		MGT.messageToUnit(unitName,'Mark all SAMs: ON')	
	elseif setting == 5 then
		MGT.MarkAllSams	= true	
		MGT.messageToUnit(unitName,'Mark all SAMs: OFF')	
	elseif 	setting >= 10 and setting <= 17 then
		local cat = ''
		MGT.MissionCategory = setting - 10
		local o = MGT.MissionCategory
		if o == 0 then 
			cat ="Any"		
		elseif o == 1 then 
			cat ="Interception (A2A)"
		elseif o == 2 then 			
			cat ="Strike (A2G static structures)"	
		elseif o == 3 then 			
			cat ="Close Air Support(JTAC)"		
		elseif o == 4 then 			
			cat ="Interdiction (A2G units)"		
		elseif o == 5 then 			
			cat ="Anti-Ship"					
		elseif o == 6 then 			
			cat ="SEAD"						
		elseif o == 7 then 			
			cat ="CAP"							
		end
		MGT.messageToUnit(unitName,string.format ('Category: %s', cat)	)
		
	end -- main if		
		
	return true
end
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.populateLand(_templateList, _quantity, _center, _borderGrpName, _minRadius, _maxRadius, _minAngle,_maxAngle, _distanceBetween)
	local border = mist.getGroupPoints(_borderGrpName)
	--local targetZone = trigger.misc.getZone(_zoneName)
	local _action = 'clone'
	local retPos = nil
	local minRad = _minRadius
	local maxRad = _maxRadius
	local grpList = {}
	
	local nt = MGT.rowCount(_templateList)
	local n = _quantity
	

	local disperse 
	local maxdis 

	
	while n > 0 and nt > 0 do
		--env.error('populateLand() quantity:'.. n,false)
		retPos = nil
		temp = _templateList[mist.random(1,nt)]
		disperse =  temp.disperse > 0
		maxdis = temp.disperse 
		local placed = false
		local attempts = 50
		local fail_v = 0
		local fail_f = 0
		local fail_i = 0
		local fail_c = 0
		local fail_e = 0
		while not placed and attempts > 0 do
		--	env.error('populateLand() attempt:'.. attempts,false)
			attempts = attempts - 1
			local rp 
			if _minAngle == _maxAngle then
				rp = mist.getRandPointInCircle(_center ,maxRad , minRad)
			else
				rp = mist.getRandPointInCircle(_center ,maxRad , minRad ,_maxAngle ,_minAngle )
			end
			local inside =  mist.pointInPolygon(rp,border)
			if inside then
				--env.error('populateLand() inside',false)
				if mist.isTerrainValid(rp, {'LAND','ROAD'}) == true then 
					--env.error('populateLand() land',false)
					if (temp.size == 0) or (mist.terrainHeightDiff(rp,temp.size) < math.floor(0.4 * temp.size)) then
						--env.error('populateLand() flat',false)
						if MGT.isAreaEmpty(rp,temp.size) then
							--env.error('populateLand() empty',false)
							local objs = MGT.populatedUnitPositions
							local good = true
							local num_objs =  MGT.rowCount(objs)	
							if num_objs > 0 then
								local i = 1
								while i <= num_objs and good do
								
									local d = mist.utils.get2DDist(rp,objs[i])
									if d <= _distanceBetween then -- distance between units
										good = false
									end
																				
									i = i + 1
								end -- while								
							end
									
							if good then
								--env.error('populateLand() good',false)
								table.insert(MGT.populatedUnitPositions, rp) 
								retPos = rp
								rp = mist.utils.makeVec3GL(rp)
								local gn = string.format('%s_%s',temp.groupName,MGT.getNextID())
								local newgrp = mist.teleportToPoint({
									groupName = temp.groupName, 
									point = rp, 
									action = _action,
									newGroupName = gn,
									disperse = disperse,
									maxDisp = maxdis,
									offsetRoute = true
								})
								if newgrp then
									local g = Group.getByName(gn)
									g:activate()
									local row = {name = gn, pos = retPos, template = temp}
									table.insert(grpList, row)
									placed = true
									n = n - 1
									env.error(string.format('populateLand() : N:%s, S:%s, A:%s -> deployed as %s',temp.groupName,temp.size,attempts,gn),false)
								end
							else -- too close
								fail_c = fail_c + 1
							end
						else -- if MGT.isAreaEmpty() then
							fail_e = fail_e + 1
							--env.error(string.format('populateLand() : N:%s, S:%s, A:%s -> Area not empty',temp.groupName,temp.size,attempts))
						end -- if MGT.isAreaEmpty() then
					else -- if mist.terrainHeightDiff(rp,10) 
						fail_f = fail_f + 1
						--env.error(string.format('populateLand() : N:%s, S:%s, A:%s -> Area not flat',temp.groupName,temp.size,attempts))
					end -- if mist.terrainHeightDiff(rp,10) 
				else  -- if mist.isTerrainValid(rp, {'LAND'}) 
					fail_v = fail_v + 1
					--env.error(string.format('populateLand() : N:%s, S:%s, A:%s -> Area not valid',temp.groupName,temp.size,attempts))
				end -- if mist.isTerrainValid(rp, {'LAND'}) 
			else-- if inside
				fail_i = fail_i + 1
				--env.error(string.format('populateLand() : N:%s, S:%s, A:%s -> Area not inside',temp.groupName,temp.size,attempts))
			end -- if inside 
		end -- while surfType == 3 do
	    if attempts == 0 then
			n = n - 1
			env.error(string.format('populateLand() : N:%s, S:%s, A:%s  i:%s / v:%s / f:%s / e:%s / c:%s   -> skipped',temp.groupName,temp.size,attempts,fail_i,fail_v,fail_f,fail_e,fail_c),false)
		end
	end -- while not placed and attempts <= 5 do
	return  grpList
end
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.populateAir(_templateList, _quantity, _center, _borderGrpName, _minRadius, _maxRadius, _minAngle,_maxAngle)
	local border = mist.getGroupPoints(_borderGrpName)
	--local targetZone = trigger.misc.getZone(_zoneName)
	local retPos = nil
	local minRad = _minRadius
	local maxRad = _maxRadius
	local grpList = {}
	
	local nt = MGT.rowCount(_templateList)
	local n = _quantity
	

	local disperse 
	local maxdis 

	
	while n > 0 and nt > 0 do
		retPos = nil
		temp = _templateList[mist.random(1,nt)]
		disperse =  temp.disperse > 0
		maxdis = temp.disperse 
		local placed = false
		local attempts = 20
		while not placed and attempts > 0 do
			attempts = attempts - 1
			local rp 
			if _minAngle == _maxAngle then
				rp = mist.getRandPointInCircle(_center ,maxRad , minRad)
			else
				rp = mist.getRandPointInCircle(_center ,maxRad , minRad ,_maxAngle ,_minAngle )
			end
			--surfType= land.getSurfaceType(rp)
			local inside =  mist.pointInPolygon(rp,border)

			if inside then
				retPos = rp
				local gn = string.format('%s_%s',temp.groupName,MGT.getNextID())
				local newgrp = mist.teleportToPoint({
					groupName = temp.groupName, 
					point = rp, 
					action = 'clone',
					newGroupName = gn,
					disperse = disperse,
					maxDisp = maxdis,
					offsetRoute = true
				})
				if newgrp then
					local g = Group.getByName(gn)
					g:activate()
					local row = {name = gn, pos = retPos, template = temp}
					table.insert(grpList, row)
					placed = true
					n = n - 1
				end

			
			end -- if inside 
		end --while not placed and attempts > 0 do
	    if attempts == 0 then
			n = n - 1
		end
	end --while n > 0 do
	return  grpList
end
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.getNextID()
    local id = 0
	
	-- Try MOOSE
	--local _status,id =  pcall(function()
	--						return {UTILS.GetMarkID()}
	--					end)	
	--if not _status then
		-- Try MIST
	--	local _status,id =  pcall(function()
	--							return {mist.marker.getNextId()}
	--						end)	
	--	if not _status then	
			-- Use MGT
			MGT.globalConfig.nextID = MGT.globalConfig.nextID+1
			id = MGT.globalConfig.nextID 
	--	end -- MIST
	
--	end -- MOOSE

	return  id -- MGT.globalConfig.nextID
end -- function
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.isAreaEmpty(_center,_radius)
	local ret = true
	if _radius < 35 then
		_radius = 35
	end
	
	local foundObjs = {}
	
	local volS = {
	   id = world.VolumeType.SPHERE,
	   params = {
		 point = _center,
		 radius = _radius
	   }
	 }
	local ifFound = 
		function(foundItem, val)
			foundObjs[#foundObjs + 1] = foundItem
			return true
		end

	world.searchObjects(Object.Category.SCENERY, volS, ifFound)
	ret = MGT.rowCount(foundObjs) == 0 

	return ret
end -- function
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.addTemplate(_groupName, _displayName, _category, _side, _era, _disperse, _size, _afacCat, _afacPri, _EWR, _C3,	_IADS, _ELINT)
	--local n = MGT.rowCount(MGT.templateList)
	--n = n + 1
	local row = {
				--	id			= n
				--,	
					groupName 	= _groupName
				,	displayName	= _displayName
				,	category	= _category
				,	side 		= _side
				,	era 		= _era
				,	disperse 	= _disperse
				, 	size		= _size
				,	afacCat		= _afacCat
				,	afacPri		= _afacPri
				,	EWR			= _EWR  
				,	C3			= _C3  
				,	IADS		= _IADS
				,	ELINT		= _ELINT 
			}		
	
	--MGT.templateList[n] = row
	MGT.templateList[_groupName] = row

end --function
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.getTemplateList (_categoryTable, _side, _era)
	local list = {}
	local n = 0
	for id,row in pairs(MGT.templateList) do
		if MGT.inTable(row.category,_categoryTable) or (_categoryTable == {}) then
			if (row.side == _side) then
				if (row.era == _era) or (row.era == 0)  then
					n = n + 1
					 list[n] = row
				end --if (row.era == _era) or (_era == nil) then
			end-- if (row.side == _side) or (_side == nil) then
		end -- if MGT.inTable(row.category,_categoryTable) or (_categoryTable = {}) then
	end -- for id,row in MGT.templateList do
	if n == 0 then
		env.error('MGT.getTemplateList: search returned 0 values!',false)
	end
	return list, n
end -- function
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.isGroupAlive(_groupName)
	local r = false
	local g = Group.getByName(_groupName)
	if g then
		for _,unit in pairs(g:getUnits()) do
			if unit:getLife() >= 1 then
				r = true
				break
			end
		end --for _,unit in pairs(g) do
	end --if g then
	return r
end -- function
-----------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.getUnitNames(_groupName)
	local list = {}
	local g = Group.getByName(_groupName)
	for _,unit in pairs(g:getUnits()) do
		table.insert(list,unit:getName() )
	end --for _,unit in pairs(g) do	
	return list
end --function
-----------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.setGroupOption(_groupName, _option,_value) 
	local con = Group.getByName(_groupName):getController()
	con:setOption(_option, _value) --hold fire
end -- funtion
-----------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.getGroupHealth(_groupName)
	local h = 0
	local n = 0
	local g = Group.getByName(_groupName)
	
	if g then
		local ut = g:getUnits()
		for index, data in pairs(ut) do
			h = h + Unit.getLife(data) 	
		end
		n = MGT.rowCount(ut)
	end
	return h,n
end-- function
-----------------------------------------------------------------------------------------------------------------------------------------------------
--
--#################################################################################################################################################
-- #############   Utility	 																										############### 
--#################################################################################################################################################
--
function MGT.messageToUnit(_unitName,_messageText,_soundTable,_displayTime)
	if _displayTime == nil then
		_displayTime = 5
	end 
	local _unit = Unit.getByName(_unitName)
	local _groupId = nil 
	local _callSign = nil 
	local _id = nil
	if _unit then
		 _groupId = _unit:getGroup():getID()
		 _callSign = _unit:getCallsign()
		 _id = _unit:getID()
	end
	if MGT.messageBeep ~= '' then
		--if _unit then
		--	trigger.action.outSound(MGT.messageBeep)
		--else
			if _id then 
				trigger.action.outSoundForUnit(_id, MGT.messageBeep)
			else
				trigger.action.outSound(MGT.messageBeep)
			end
		--end
	end
	local _msg = _messageText
	if _id then 
		trigger.action.outTextForUnit(_id, _msg, _displayTime)
	else
		trigger.action.outText( _msg, _displayTime)
	end
	--

	
	return true
end
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.messageToGroup(_groupName,_messageText,_soundTable,_displayTime)
	if _displayTime == nil then
		_displayTime = 5
	end 
	local _group = Group.getByName(_groupName)
	local _groupId = nil 
	local _callSign = nil 

	if _group then
		 _groupId = _group:getID()
		 local _units = _group:getUnits()
		 _callSign = _units[1]:getCallsign()

	end
	if MGT.messageBeep ~= '' then
		--if _unit then
		--	trigger.action.outSound(MGT.messageBeep)
		--else
			if _groupId then 
				trigger.action.outSoundForGroup(_groupId, MGT.messageBeep)
			else
				trigger.action.outSound(MGT.messageBeep)
			end
		--end
	end
	local _msg = _messageText
	if _groupId then 
		trigger.action.outTextForGroup(_groupId, _msg, _displayTime)
	else
		trigger.action.outText( _msg, _displayTime)
	end
	--

	
	return true
end
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.convertDistance(_unitName,_distance)
	local s
	local u
	local cfg = MGT.unitConfig[_unitName] 
	-- 1 = Nautical Miles, 2 = Miles, 3 = Kilometers
	-- _distance in meters
	if cfg.DistanceUOM == 1 then
		s = _distance * 0.000539957						
		u = 'NM'
	elseif cfg.DistanceUOM == 2 then
		s = _distance * 0.000621371
		u = 'mi'
	else
		s = _distance * 0.001
		u = 'Km'
	end
	return s,u
end
-----------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.convertAltitude(_unitName,_altitude)
	local s
	local u
	local cfg = MGT.unitConfig[_unitName] 
	-- 1 = feet, 2 = yards, 3 = meters
	-- _distance in meters
	if cfg.AltitudeUOM == 1 then
		s = _altitude * 3.28084
		u = 'ft'
	else
		s = _altitude 
		u = 'm'
	end
	return s,u
end
-----------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.getDirection(_point1, _point2)
    local tgtBearing
    local xUnit = _point1.x
    local yUnit = _point1.z
    local xZone = _point2.x
    local yZone = _point2.z
    
    local xDiff = xUnit - xZone
    local yDiff = yUnit - yZone
    
    local tgtAngle = math.deg(math.atan(yDiff/xDiff))
    
    if xDiff > 0 then 
    tgtBearing = 180 + tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle > 0 then
    tgtBearing = tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle < 0 then
    tgtBearing = 360 + tgtAngle
    end
    
   
    return tgtBearing
end
-----------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.rowCount(t)
	local r = 0

		if next(t) == nil then
			return r
		elseif t == {} then
			return r
		else
			if t then
				for _,_ in pairs(t) do
					r = r + 1
				end
			end
		end

	return r
end -- function
-----------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.inTable(v,t)
	local r = true
	for _,j in pairs(t) do
		if j == v then
			return true
		end
	end
	return false
	
end -- function
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.getCoords(unitName,pos)
	local lat, lon, alt = coord.LOtoLL(pos)

	--MGRS
	local _coord_mgrs = mist.tostringMGRS(coord.LLtoMGRS(coord.LOtoLL(pos)), 5):gsub(string.char(9),'   ')
	-- DMS
	local _coord_dms = mist.tostringLL(lat, lon, 3, true):gsub(string.char(9),'   ')	
	-- DM.mm
	local _coord_dm = mist.tostringLL(lat, lon, 4):gsub(string.char(9),'   ')
	local a,auom = MGT.convertAltitude(unitName,alt)
	return _coord_mgrs,_coord_dms,_coord_dm, a, auom
end -- function
-----------------------------------------------------------------------------------------------------------------------------------------------------
function MGT.tableConcat(t1,t2)
   for i=1,#t2 do
      t1[#t1+1] = t2[i]
   end
   return t1
end
-----------------------------------------------------------------------------------------------------------------------------------------------------
--
--#################################################################################################################################################
-- #############   Main body   																										############### 
--#################################################################################################################################################
--
do

	trigger.action.outText( 'MGT - v'..MGT.Version, 1 )
end






